@@ -5,6 +5,8 @@ import numpy as np
 import torch.nn as nn
 import torch.nn.functional as F
 
+from abc import abstractmethod
+from abc import ABCMeta
 from torch import Tensor
 from typing import Any
 from typing import Dict
@@ -17,11 +19,13 @@ from torch.nn import Module
 from cftool.misc import safe_execute
 from cftool.misc import WithRegister
 
+from .convs import conv_nd
 from .convs import Conv2d
-from .utils import conv_nd
+from .convs import HijackConv2d
 from .utils import zero_module
 from .common import Lambda
 from .customs import Linear
+from .customs import HijackLinear
 from .activations import Activation
 from ...misc.toolkit import gradient_checkpoint
 
@@ -34,6 +38,19 @@ class AttentionOutput(NamedTuple):
     weights: Tensor
 
 
+qkv_type = Tuple[Tensor, Tensor, Tensor]
+
+
+class AttentionHook(Module, metaclass=ABCMeta):
+    @abstractmethod
+    def after_attn(self, qkv_inp: qkv_type, qkv_out: qkv_type) -> qkv_type:
+        pass
+
+    @abstractmethod
+    def after_out(self, inp: Tensor, out: Tensor) -> Tensor:
+        pass
+
+
 class Attention(Module, WithRegister["Attention"]):
     d = attentions
 
@@ -55,6 +72,7 @@ class Attention(Module, WithRegister["Attention"]):
         activation_config: Optional[Dict[str, Any]] = None,
         out_linear_config: Optional[Dict[str, Any]] = None,
         reduction_ratio: Optional[int] = None,
+        hook: Optional[AttentionHook] = None,
     ):
         super().__init__()
         self.input_dim = input_dim
@@ -139,6 +157,8 @@ class Attention(Module, WithRegister["Attention"]):
                 nn.LayerNorm(self.embed_dim),
             )
 
+        self.hook = hook
+
     def _to_heads(self, tensor: Tensor, determinate: bool) -> Tensor:
         seq_len = tensor.shape[1]
         if determinate:
@@ -174,6 +194,7 @@ class Attention(Module, WithRegister["Attention"]):
         mask: Optional[Tensor] = None,
         determinate: bool = False,
     ) -> AttentionOutput:
+        qkv_inp = q, k, v
         # `mask` represents slots which will be zeroed
         if self.qkv_same:
             qkv = F.linear(q, self.in_w, self.qkv_bias)
@@ -205,6 +226,8 @@ class Attention(Module, WithRegister["Attention"]):
             k = F.linear(self._reduce(k, hw), self.k_w, k_bias)
             # B, Nv, Dv -> B, Nv, D
             v = F.linear(self._reduce(v, hw), self.v_w, v_bias)
+        if self.hook is not None:
+            q, k, v = self.hook.after_attn(qkv_inp, (q, k, v))
         q, k, v = map(self.activation, [q, k, v])
         # B, N*, D -> B * N_head, N*, D_head
         q, k, v = map(self._to_heads, [q, k, v], [determinate] * 3)
@@ -233,8 +256,11 @@ class Attention(Module, WithRegister["Attention"]):
             seq_len = int(seq_len)
         output = output.view(-1, seq_len, self.embed_dim)
         # B, Nq, D -> B, Nq, Din
-        output = self.activation(self.out_linear(output))
-        return AttentionOutput(output, weights)
+        net = self.out_linear(output)
+        if self.hook is not None:
+            net = self.hook.after_out(output, net)
+        net = self.activation(net)
+        return AttentionOutput(net, weights)
 
 
 Attention.register("basic")(Attention)
@@ -302,10 +328,10 @@ class SpatialAttention(Module):
             eps=eps,
             affine=True,
         )
-        self.to_q = nn.Conv2d(in_channels, in_channels, 1, 1, 0)
-        self.to_k = nn.Conv2d(in_channels, in_channels, 1, 1, 0)
-        self.to_v = nn.Conv2d(in_channels, in_channels, 1, 1, 0)
-        self.to_out = nn.Conv2d(in_channels, in_channels, 1, 1, 0)
+        self.to_q = HijackConv2d(in_channels, in_channels, 1, 1, 0)
+        self.to_k = HijackConv2d(in_channels, in_channels, 1, 1, 0)
+        self.to_v = HijackConv2d(in_channels, in_channels, 1, 1, 0)
+        self.to_out = HijackConv2d(in_channels, in_channels, 1, 1, 0)
 
     def forward(self, net: Tensor) -> Tensor:
         inp = net
@@ -435,8 +461,8 @@ class LinearDepthWiseAttention(Module):
         self.num_heads = num_heads
         self.head_dim = head_dim
         self.latent_dim = num_heads * head_dim
-        self.to_qkv = nn.Conv2d(in_channels, self.latent_dim * 3, 1, bias=False)
-        self.to_out = nn.Conv2d(self.latent_dim, in_channels, 1)
+        self.to_qkv = HijackConv2d(in_channels, self.latent_dim * 3, 1, bias=False)
+        self.to_out = HijackConv2d(self.latent_dim, in_channels, 1)
 
     def transpose(self, net: Tensor) -> Tensor:
         # (B, H * W, C) -> (B, H * W, head, dim)
@@ -478,12 +504,12 @@ class CrossAttention(Module):
         self.num_heads = num_heads
         self.attn_split_chunk = attn_split_chunk
 
-        self.to_q = nn.Linear(query_dim, latent_dim, bias=False)
-        self.to_k = nn.Linear(context_dim, latent_dim, bias=False)
-        self.to_v = nn.Linear(context_dim, latent_dim, bias=False)
+        self.to_q = HijackLinear(query_dim, latent_dim, bias=False)
+        self.to_k = HijackLinear(context_dim, latent_dim, bias=False)
+        self.to_v = HijackLinear(context_dim, latent_dim, bias=False)
 
         self.out_linear = nn.Sequential(
-            nn.Linear(latent_dim, query_dim),
+            HijackLinear(latent_dim, query_dim),
             nn.Dropout(dropout),
         )
 
